@@ -4,6 +4,7 @@ import imghdr
 import requests
 
 from django import forms
+from django.core.exceptions import ValidationError
 from django.core.validators import MaxValueValidator, MinValueValidator
 from django.utils import timezone
 from django.utils.html import mark_safe
@@ -14,7 +15,7 @@ from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
 from allianceauth.services.hooks import get_extension_logger
 from app_utils.logging import LoggerAddTag
 
-from . import __title__
+from . import __title__, constants
 from .models import Timer
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
@@ -48,16 +49,16 @@ class TimerForm(forms.ModelForm):
             # for appropriate fields
             my_instance = kwargs["instance"]
             current_time = timezone.now()
-            td = my_instance.date - current_time
-            initial = kwargs.pop("initial", dict())
-            if "days_left" not in initial:
-                initial.update({"days_left": td.days})
-            if "hours_left" not in initial:
-                initial.update({"hours_left": td.seconds // 3600})
-            if "minutes_left" not in initial:
-                initial.update({"minutes_left": td.seconds // 60 % 60})
-
-            kwargs.update({"initial": initial})
+            if my_instance.date:
+                td = my_instance.date - current_time
+                initial = kwargs.pop("initial", dict())
+                if "days_left" not in initial:
+                    initial.update({"days_left": td.days})
+                if "hours_left" not in initial:
+                    initial.update({"hours_left": td.seconds // 3600})
+                if "minutes_left" not in initial:
+                    initial.update({"minutes_left": td.seconds // 60 % 60})
+                kwargs.update({"initial": initial})
             self.is_new = False
         else:
             my_instance = None
@@ -91,16 +92,16 @@ class TimerForm(forms.ModelForm):
         widget=forms.Select(attrs={"class": "select2-structure-types"}),
     )
     objective = forms.ChoiceField(
-        initial=Timer.OBJECTIVE_UNDEFINED,
-        choices=Timer.OBJECTIVE_CHOICES,
+        initial=Timer.Objective.UNDEFINED,
+        choices=Timer.Objective.choices,
         widget=forms.Select(attrs={"class": "select2-render"}),
     )
     timer_type = forms.ChoiceField(
-        choices=Timer.TYPE_CHOICES,
+        choices=Timer.Type.choices,
         widget=forms.Select(attrs={"class": "select2-render"}),
     )
     visibility = forms.ChoiceField(
-        choices=Timer.VISIBILITY_CHOICES,
+        choices=Timer.Visibility.choices,
         widget=forms.Select(attrs={"class": "select2-render"}),
     )
     days_left = forms.IntegerField(
@@ -151,6 +152,17 @@ class TimerForm(forms.ModelForm):
                         structure_type.name,
                     )
                 ]
+                if (
+                    cleaned_data.get("timer_type") == Timer.Type.MOONMINING
+                    and structure_type.eve_group_id != constants.EVE_GROUP_ID_REFINERY
+                ):
+                    raise ValidationError(
+                        {
+                            "timer_type": _(
+                                "Moon mining timers are valid for refineries only."
+                            )
+                        }
+                    )
 
         if cleaned_data.get("details_image_url"):
             details_image_url = cleaned_data["details_image_url"]
@@ -192,7 +204,7 @@ class TimerForm(forms.ModelForm):
                     )
 
     def save(self, commit=True):
-        timer = super(TimerForm, self).save(commit=False)
+        timer = super().save(commit=False)
 
         # character / corporation / alliance
         if self.is_new:
