@@ -10,6 +10,7 @@ from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.db import models
 from django.utils.translation import gettext_lazy as _
+from eveuniverse.helpers import meters_to_ly
 from eveuniverse.models import EveSolarSystem, EveType
 
 from allianceauth.eveonline.evelinks import dotlan
@@ -25,15 +26,36 @@ from app_utils.logging import LoggerAddTag
 from app_utils.urls import reverse_absolute, static_file_absolute_url
 
 from . import __title__
-from .app_settings import STRUCTURETIMERS_NOTIFICATIONS_ENABLED
-from .managers import NotificationRuleManager, TimerManager
+from .app_settings import (
+    STRUCTURETIMER_NOTIFICATION_SET_AVATAR,
+    STRUCTURETIMERS_NOTIFICATIONS_ENABLED,
+)
+from .managers import DistancesFromStagingManager, NotificationRuleManager, TimerManager
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
 
 def default_avatar_url() -> str:
     """avatar url for all messages"""
-    return static_file_absolute_url("structuretimers/structuretimers_logo.png")
+    return static_file_absolute_url("structuretimers/img/structuretimers_logo.png")
+
+
+def _task_calc_staging_system():
+    from .tasks import calc_staging_system
+
+    return calc_staging_system
+
+
+def _task_calc_timer_distances_for_all_staging_systems():
+    from .tasks import calc_timer_distances_for_all_staging_systems
+
+    return calc_timer_distances_for_all_staging_systems
+
+
+def _task_schedule_notifications_for_timer():
+    from .tasks import schedule_notifications_for_timer
+
+    return schedule_notifications_for_timer
 
 
 class General(models.Model):
@@ -249,7 +271,9 @@ class DiscordWebhook(models.Model):
 class Timer(models.Model):
     """A structure timer"""
 
-    # timer type
+    # TODO: Old constants needed to maintain compatibility with other apps
+    # during transition only. REMOVE as soon as possible.
+
     TYPE_NONE = "NO"
     TYPE_ARMOR = "AR"
     TYPE_HULL = "HL"
@@ -258,130 +282,39 @@ class Timer(models.Model):
     TYPE_UNANCHORING = "UA"
     TYPE_MOONMINING = "MM"
 
-    TYPE_CHOICES = (
-        (TYPE_NONE, _("Unspecified")),
-        (TYPE_ARMOR, _("Armor")),
-        (TYPE_HULL, _("Hull")),
-        (TYPE_FINAL, _("Final")),
-        (TYPE_ANCHORING, _("Anchoring")),
-        (TYPE_UNANCHORING, _("Unanchoring")),
-        (TYPE_MOONMINING, _("Moon Mining")),
-    )
-
-    # objective
     OBJECTIVE_UNDEFINED = "UN"
     OBJECTIVE_HOSTILE = "HO"
     OBJECTIVE_FRIENDLY = "FR"
     OBJECTIVE_NEUTRAL = "NE"
 
-    OBJECTIVE_CHOICES = [
-        (OBJECTIVE_UNDEFINED, _("undefined")),
-        (OBJECTIVE_HOSTILE, _("hostile")),
-        (OBJECTIVE_FRIENDLY, _("friendly")),
-        (OBJECTIVE_NEUTRAL, _("neutral")),
-    ]
-
-    # visibility
     VISIBILITY_UNRESTRICTED = "UN"
     VISIBILITY_ALLIANCE = "AL"
     VISIBILITY_CORPORATION = "CO"
 
-    VISIBILITY_CHOICES = [
-        (VISIBILITY_UNRESTRICTED, _("unrestricted")),
-        (VISIBILITY_ALLIANCE, _("Alliance only")),
-        (VISIBILITY_CORPORATION, _("Corporation only")),
-    ]
+    class Type(models.TextChoices):
+        NONE = "NO", _("Unspecified")
+        ARMOR = "AR", _("Armor")
+        HULL = "HL", _("Hull")
+        FINAL = "FI", _("Final")
+        ANCHORING = "AN", _("Anchoring")
+        UNANCHORING = "UA", _("Unanchoring")
+        MOONMINING = "MM", _("Moon Mining")
+
+    class Objective(models.TextChoices):
+        UNDEFINED = "UN", _("undefined")
+        HOSTILE = "HO", _("hostile")
+        FRIENDLY = "FR", _("friendly")
+        NEUTRAL = "NE", _("neutral")
+
+    class Visibility(models.TextChoices):
+        UNRESTRICTED = "UN", _("unrestricted")
+        ALLIANCE = "AL", _("Alliance only")
+        CORPORATION = "CO", _("Corporation only")
 
-    timer_type = models.CharField(max_length=2, choices=TYPE_CHOICES, default=TYPE_NONE)
-    eve_solar_system = models.ForeignKey(
-        EveSolarSystem, on_delete=models.CASCADE, default=None, null=True
-    )
-    location_details = models.CharField(
-        max_length=254,
-        default="",
-        blank=True,
-        help_text=(
-            "Additional information about the location of this structure, "
-            "e.g. name of nearby planet / moon / gate"
-        ),
-    )
-    structure_type = models.ForeignKey(
-        EveType,
-        on_delete=models.CASCADE,
-    )
-    structure_name = models.CharField(max_length=254, default="", blank=True)
-    objective = models.CharField(
-        max_length=2,
-        choices=OBJECTIVE_CHOICES,
-        default=OBJECTIVE_UNDEFINED,
-    )
     date = models.DateTimeField(
         db_index=True,
         help_text="Date when this timer happens",
     )
-    is_important = models.BooleanField(
-        default=False,
-        help_text="Mark this timer as is_important",
-    )
-    owner_name = models.CharField(
-        max_length=254,
-        default=None,
-        blank=True,
-        null=True,
-        help_text="Name of the corporation owning the structure",
-    )
-    is_opsec = models.BooleanField(
-        default=False,
-        db_index=True,
-        help_text=(
-            "Limit access to users with OPSEC clearance. "
-            "Can be combined with visibility."
-        ),
-    )
-    eve_character = models.ForeignKey(
-        EveCharacter,
-        on_delete=models.SET_DEFAULT,
-        default=None,
-        null=True,
-        blank=True,
-        related_name="Timers",
-        help_text="Main character of the user who created this timer",
-    )
-    eve_corporation = models.ForeignKey(
-        EveCorporationInfo,
-        on_delete=models.SET_DEFAULT,
-        default=None,
-        null=True,
-        blank=True,
-        related_name="Timers",
-        help_text="Corporation of the user who created this timer",
-    )
-    eve_alliance = models.ForeignKey(
-        EveAllianceInfo,
-        on_delete=models.SET_DEFAULT,
-        default=None,
-        null=True,
-        blank=True,
-        related_name="Timers",
-        help_text="Alliance of the user who created this timer",
-    )
-    visibility = models.CharField(
-        max_length=2,
-        choices=VISIBILITY_CHOICES,
-        default=VISIBILITY_UNRESTRICTED,
-        db_index=True,
-        help_text=(
-            "The visibility of this timer can be limited to members"
-            " of your organization"
-        ),
-    )
-    user = models.ForeignKey(
-        User,
-        null=True,
-        on_delete=models.SET_NULL,
-        blank=True,
-        related_name="Timers",
-    )
     details_image_url = models.CharField(
         max_length=1024,
         default=None,
@@ -397,12 +330,99 @@ class Timer(models.Model):
         blank=True,
         help_text="Notes with additional information about this timer",
     )
+    eve_alliance = models.ForeignKey(
+        EveAllianceInfo,
+        on_delete=models.SET_DEFAULT,
+        default=None,
+        null=True,
+        blank=True,
+        related_name="+",
+        help_text="Alliance of the user who created this timer",
+    )
+    eve_character = models.ForeignKey(
+        EveCharacter,
+        on_delete=models.SET_DEFAULT,
+        default=None,
+        null=True,
+        blank=True,
+        related_name="+",
+        help_text="Main character of the user who created this timer",
+    )
+    eve_corporation = models.ForeignKey(
+        EveCorporationInfo,
+        on_delete=models.SET_DEFAULT,
+        default=None,
+        null=True,
+        blank=True,
+        related_name="+",
+        help_text="Corporation of the user who created this timer",
+    )
+    eve_solar_system = models.ForeignKey(
+        EveSolarSystem,
+        on_delete=models.CASCADE,
+        default=None,
+        null=True,
+        related_name="+",
+    )
+    is_important = models.BooleanField(
+        default=False,
+        help_text="Mark this timer as is_important",
+    )
+    is_opsec = models.BooleanField(
+        default=False,
+        db_index=True,
+        help_text=(
+            "Limit access to users with OPSEC clearance. "
+            "Can be combined with visibility."
+        ),
+    )
+    location_details = models.CharField(
+        max_length=254,
+        default="",
+        blank=True,
+        help_text=(
+            "Additional information about the location of this structure, "
+            "e.g. name of nearby planet / moon / gate"
+        ),
+    )
     notification_rules = models.ManyToManyField(
         "NotificationRule",
         through="ScheduledNotification",
         through_fields=("timer", "notification_rule"),
         help_text="Notification rules conforming with this timer",
     )
+    objective = models.CharField(
+        max_length=2, choices=Objective.choices, default=Objective.UNDEFINED
+    )
+    owner_name = models.CharField(
+        max_length=254,
+        default=None,
+        blank=True,
+        null=True,
+        help_text="Name of the corporation owning the structure",
+    )
+    structure_type = models.ForeignKey(
+        EveType, on_delete=models.CASCADE, related_name="+"
+    )
+    structure_name = models.CharField(max_length=254, default="", blank=True)
+    timer_type = models.CharField(max_length=2, choices=Type.choices, default=Type.NONE)
+    user = models.ForeignKey(
+        User,
+        null=True,
+        on_delete=models.SET_NULL,
+        blank=True,
+        related_name="+",
+    )
+    visibility = models.CharField(
+        max_length=2,
+        choices=Visibility.choices,
+        default=Visibility.UNRESTRICTED,
+        db_index=True,
+        help_text=(
+            "The visibility of this timer can be limited to members"
+            " of your organization"
+        ),
+    )
 
     objects = TimerManager()
 
@@ -423,30 +443,29 @@ class Timer(models.Model):
             disable_notifications = kwargs.pop("disable_notifications")
         except KeyError:
             disable_notifications = False
-
         notification_enabled = (
             STRUCTURETIMERS_NOTIFICATIONS_ENABLED and not disable_notifications
         )
+        try:
+            old_instance = Timer.objects.get(pk=self.pk)
+        except (Timer.DoesNotExist, ValueError):
+            needs_recalc = True
+            date_changed = False
+        else:
+            needs_recalc = self.eve_solar_system != old_instance.eve_solar_system
+            date_changed = self.date != old_instance.date
         is_new = self.pk is None
-        date_changed = False
-        if notification_enabled and not is_new:
-            try:
-                date_changed = self.date != Timer.objects.get(pk=self.pk).date
-            except Timer.DoesNotExist:
-                pass
-
         super().save(*args, **kwargs)
+        if needs_recalc:
+            self.distances.all().delete()
+            _task_calc_timer_distances_for_all_staging_systems().apply_async(
+                args=[self.pk], priority=4
+            )
         if notification_enabled and (is_new or date_changed):
-            self._import_schedule_notifications_for_timer().apply_async(
+            _task_schedule_notifications_for_timer().apply_async(
                 kwargs={"timer_pk": self.pk, "is_new": is_new}, priority=3
             )
 
-    @staticmethod
-    def _import_schedule_notifications_for_timer() -> object:
-        from .tasks import schedule_notifications_for_timer
-
-        return schedule_notifications_for_timer
-
     @property
     def structure_display_name(self):
         return "{}{} in {}{}".format(
@@ -472,7 +491,7 @@ class Timer(models.Model):
             return False
 
         if (
-            self.visibility == self.VISIBILITY_CORPORATION
+            self.visibility == self.Visibility.CORPORATION
             and self.eve_corporation
             and (
                 not user.profile.main_character
@@ -485,7 +504,7 @@ class Timer(models.Model):
             return False
 
         if (
-            self.visibility == self.VISIBILITY_ALLIANCE
+            self.visibility == self.Visibility.ALLIANCE
             and self.eve_alliance
             and (
                 not user.profile.main_character
@@ -503,13 +522,13 @@ class Timer(models.Model):
     def label_type_for_timer_type(self) -> str:
         """returns the Boostrap label type for a timer_type"""
         label_types_map = {
-            self.TYPE_NONE: "default",
-            self.TYPE_ARMOR: "danger",
-            self.TYPE_HULL: "danger",
-            self.TYPE_FINAL: "danger",
-            self.TYPE_ANCHORING: "warning",
-            self.TYPE_UNANCHORING: "warning",
-            self.TYPE_MOONMINING: "success",
+            self.Type.NONE: "default",
+            self.Type.ARMOR: "danger",
+            self.Type.HULL: "danger",
+            self.Type.FINAL: "danger",
+            self.Type.ANCHORING: "warning",
+            self.Type.UNANCHORING: "warning",
+            self.Type.MOONMINING: "success",
         }
         if self.timer_type in label_types_map:
             label_type = label_types_map[self.timer_type]
@@ -520,10 +539,10 @@ class Timer(models.Model):
     def label_type_for_objective(self) -> str:
         """returns the Boostrap label type for objective"""
         label_types_map = {
-            self.OBJECTIVE_FRIENDLY: "primary",
-            self.OBJECTIVE_HOSTILE: "danger",
-            self.OBJECTIVE_NEUTRAL: "info",
-            self.OBJECTIVE_UNDEFINED: "default",
+            self.Objective.FRIENDLY: "primary",
+            self.Objective.HOSTILE: "danger",
+            self.Objective.NEUTRAL: "info",
+            self.Objective.UNDEFINED: "default",
         }
         if self.objective in label_types_map:
             label_type = label_types_map[self.objective]
@@ -532,7 +551,7 @@ class Timer(models.Model):
         return label_type
 
     def send_notification(self, webhook: DiscordWebhook, content: str = None) -> None:
-        """sends notification for given self to given webhook"""
+        """Sends notification related to this timer to given webhook."""
         structure_type_name = self.structure_type.name
         solar_system_name = self.eve_solar_system.name
         title = f"{structure_type_name} in {solar_system_name}"
@@ -556,9 +575,9 @@ class Timer(models.Model):
             f"Our stance is: **{self.get_objective_display()}**."
         )
         structure_icon_url = self.structure_type.icon_url(size=128)
-        if self.objective == self.OBJECTIVE_FRIENDLY:
+        if self.objective == self.Objective.FRIENDLY:
             color = int("0x375a7f", 16)
-        elif self.objective == self.OBJECTIVE_HOSTILE:
+        elif self.objective == self.Objective.HOSTILE:
             color = int("0xd9534f", 16)
         else:
             color = None
@@ -570,23 +589,26 @@ class Timer(models.Model):
             thumbnail=dhooks_lite.Thumbnail(structure_icon_url),
             color=color,
         )
+        if STRUCTURETIMER_NOTIFICATION_SET_AVATAR:
+            username = __title__
+            avatar_url = default_avatar_url()
+        else:
+            username = None
+            avatar_url = None
         webhook.send_message(
             content=content,
             embeds=[embed],
-            username=__title__,
-            avatar_url=default_avatar_url(),
+            username=username,
+            avatar_url=avatar_url,
         )
 
 
 class NotificationRule(models.Model):
 
     # Trigger choices
-    TRIGGER_NEW_TIMER_CREATED = "TC"
-    TRIGGER_SCHEDULED_TIME_REACHED = "TR"
-    TRIGGER_CHOICES = (
-        (TRIGGER_NEW_TIMER_CREATED, _("New timer created")),
-        (TRIGGER_SCHEDULED_TIME_REACHED, _("Scheduled time reached")),
-    )
+    class Trigger(models.TextChoices):
+        NEW_TIMER_CREATED = "TC", _("New timer created")
+        SCHEDULED_TIME_REACHED = "TR", _("Scheduled time reached")
 
     # Minutes choices
     MINUTES_0 = 0
@@ -611,29 +633,27 @@ class NotificationRule(models.Model):
         (MINUTES_120, _("T - 120 minutes")),
     )
 
-    # Ping type choices
-    PING_TYPE_NONE = "PN"
-    PING_TYPE_HERE = "PH"
-    PING_TYPE_EVERYONE = "PE"
-    PING_TYPE_CHOICES = (
-        (PING_TYPE_NONE, "(no ping)"),
-        (PING_TYPE_HERE, "@here"),
-        (PING_TYPE_EVERYONE, "@everyone"),
-    )
+    class PingType(models.TextChoices):
+        NONE = "PN", "(no ping)"
+        HERE = "PH", "@here"
+        EVERYONE = "PE", "@everyone"
 
-    # Clause choices
-    CLAUSE_ANY = "AN"
-    CLAUSE_REQUIRED = "RQ"
-    CLAUSE_EXCLUDED = "EX"
-    CLAUSE_CHOICES = (
-        (CLAUSE_ANY, "any"),
-        (CLAUSE_REQUIRED, "required"),
-        (CLAUSE_EXCLUDED, "excluded"),
-    )
+        def to_text(self) -> str:
+            """returns the text for creating the given ping on Discord"""
+            my_map = {self.NONE: "", self.HERE: "@here", self.EVERYONE: "@everyone"}
+            try:
+                return my_map[self]
+            except KeyError:
+                return ""
+
+    class Clause(models.TextChoices):
+        ANY = "AN", "any"
+        REQUIRED = "RQ", "required"
+        EXCLUDED = "EX", "excluded"
 
     trigger = models.CharField(
         max_length=2,
-        choices=TRIGGER_CHOICES,
+        choices=Trigger.choices,
         help_text="Trigger for sending a notification",
     )
     scheduled_time = models.PositiveIntegerField(
@@ -654,8 +674,8 @@ class NotificationRule(models.Model):
     )
     ping_type = models.CharField(
         max_length=2,
-        choices=PING_TYPE_CHOICES,
-        default=PING_TYPE_NONE,
+        choices=PingType.choices,
+        default=PingType.NONE,
         help_text="Options for pinging on notification",
     )
     is_enabled = models.BooleanField(
@@ -663,7 +683,7 @@ class NotificationRule(models.Model):
         help_text="whether this rule is currently active",
     )
     require_timer_types = MultiSelectField(
-        choices=Timer.TYPE_CHOICES,
+        choices=Timer.Type.choices,
         blank=True,
         help_text=(
             "Timer must have one of the given timer types "
@@ -671,12 +691,12 @@ class NotificationRule(models.Model):
         ),
     )
     exclude_timer_types = MultiSelectField(
-        choices=Timer.TYPE_CHOICES,
+        choices=Timer.Type.choices,
         blank=True,
         help_text="Timer must NOT have one of the given timer types",
     )
     require_objectives = MultiSelectField(
-        choices=Timer.OBJECTIVE_CHOICES,
+        choices=Timer.Objective.choices,
         blank=True,
         help_text=(
             "Timer must have one of the given objectives "
@@ -684,7 +704,7 @@ class NotificationRule(models.Model):
         ),
     )
     exclude_objectives = MultiSelectField(
-        choices=Timer.OBJECTIVE_CHOICES,
+        choices=Timer.Objective.choices,
         blank=True,
         help_text="Timer must NOT have one of the given objectives",
     )
@@ -719,27 +739,27 @@ class NotificationRule(models.Model):
         help_text="Timer must NOT be created by one of the given alliances",
     )
     require_visibility = MultiSelectField(
-        choices=Timer.VISIBILITY_CHOICES,
+        choices=Timer.Visibility.choices,
         blank=True,
         help_text=(
             "Visibility must be one of the selected or leave blank to match any."
         ),
     )
     exclude_visibility = MultiSelectField(
-        choices=Timer.VISIBILITY_CHOICES,
+        choices=Timer.Visibility.choices,
         blank=True,
         help_text="Visibility must NOT be one of the selected",
     )
     is_important = models.CharField(
         max_length=2,
-        choices=CLAUSE_CHOICES,
-        default=CLAUSE_ANY,
+        choices=Clause.choices,
+        default=Clause.ANY,
         help_text="Wether the timer must be important",
     )
     is_opsec = models.CharField(
         max_length=2,
-        choices=CLAUSE_CHOICES,
-        default=CLAUSE_ANY,
+        choices=Clause.choices,
+        default=Clause.ANY,
         help_text="Wether the timer must be OPSEC",
     )
 
@@ -753,14 +773,14 @@ class NotificationRule(models.Model):
         if (
             STRUCTURETIMERS_NOTIFICATIONS_ENABLED
             and self.is_enabled
-            and self.trigger == self.TRIGGER_SCHEDULED_TIME_REACHED
+            and self.trigger == self.Trigger.SCHEDULED_TIME_REACHED
         ):
             self._import_schedule_notifications_for_rule().apply_async(
                 kwargs={"notification_rule_pk": self.pk}, priority=4
             )
 
-        if self.trigger == self.TRIGGER_NEW_TIMER_CREATED:
-            ScheduledNotification.objects.filter(notification_rule=self).delete()
+        if self.trigger == self.Trigger.NEW_TIMER_CREATED:
+            self.scheduled_notifications.all().delete()
 
     @staticmethod
     def _import_schedule_notifications_for_rule() -> object:
@@ -770,7 +790,7 @@ class NotificationRule(models.Model):
 
     @property
     def ping_type_text(self) -> str:
-        return self.ping_type_to_text(self.ping_type)
+        return self.PingType(self.ping_type).to_text()
 
     def prepend_ping_text(self, text: str) -> str:
         """prepends ping text to given text and returns it as new text string"""
@@ -779,16 +799,16 @@ class NotificationRule(models.Model):
     def is_matching_timer(self, timer: "Timer") -> bool:
         """returns True if notification rule is matching the given timer, else False"""
         is_matching = True
-        if is_matching and self.is_important == self.CLAUSE_REQUIRED:
+        if is_matching and self.is_important == self.Clause.REQUIRED:
             is_matching = timer.is_important
 
-        if is_matching and self.is_important == self.CLAUSE_EXCLUDED:
+        if is_matching and self.is_important == self.Clause.EXCLUDED:
             is_matching = not timer.is_important
 
-        if is_matching and self.is_opsec == self.CLAUSE_REQUIRED:
+        if is_matching and self.is_opsec == self.Clause.REQUIRED:
             is_matching = timer.is_opsec
 
-        if is_matching and self.is_opsec == self.CLAUSE_EXCLUDED:
+        if is_matching and self.is_opsec == self.Clause.EXCLUDED:
             is_matching = not timer.is_opsec
 
         if is_matching and self.require_visibility:
@@ -823,24 +843,6 @@ class NotificationRule(models.Model):
 
         return is_matching
 
-    @classmethod
-    def ping_type_to_text(cls, ping_type: str) -> str:
-        """returns the text for creating the given ping on Discord"""
-        my_map = {
-            NotificationRule.PING_TYPE_NONE: "",
-            NotificationRule.PING_TYPE_HERE: "@here",
-            NotificationRule.PING_TYPE_EVERYONE: "@everyone",
-        }
-        return my_map[ping_type] if ping_type in my_map else ""
-
-    @classmethod
-    def get_timer_type_display(cls, value: Any) -> str:
-        return cls.get_multiselect_display(value, Timer.TYPE_CHOICES)
-
-    @classmethod
-    def get_objectives_display(cls, value: Any) -> str:
-        return cls.get_multiselect_display(value, Timer.OBJECTIVE_CHOICES)
-
     @staticmethod
     def get_multiselect_display(value: Any, choices: List[Tuple[Any, str]]) -> str:
         for choice, text in choices:
@@ -853,8 +855,15 @@ class NotificationRule(models.Model):
 class ScheduledNotification(models.Model):
     """A scheduled notification task"""
 
-    timer = models.ForeignKey(Timer, on_delete=models.CASCADE)
-    notification_rule = models.ForeignKey(NotificationRule, on_delete=models.CASCADE)
+    timer = models.ForeignKey(
+        Timer, on_delete=models.CASCADE, related_name="scheduled_notifications"
+    )
+    notification_rule = models.ForeignKey(
+        NotificationRule,
+        on_delete=models.CASCADE,
+        related_name="scheduled_notifications",
+    )
+
     timer_date = models.DateTimeField(db_index=True)
     notification_date = models.DateTimeField(db_index=True)
     celery_task_id = models.CharField(max_length=765, default="")
@@ -875,3 +884,68 @@ class ScheduledNotification(models.Model):
             f"timer_date='{self.timer_date}', "
             f"notification_date='{self.notification_date}')"
         )
+
+
+class StagingSystem(models.Model):
+    """A staging system."""
+
+    eve_solar_system = models.OneToOneField(
+        EveSolarSystem,
+        on_delete=models.SET_DEFAULT,
+        default=None,
+        null=True,
+        blank=True,
+        related_name="+",
+    )
+    is_main = models.BooleanField(default=False)
+
+    def __str__(self) -> str:
+        return str(self.eve_solar_system)
+
+    def save(self, *args, **kwargs) -> None:
+        try:
+            old_instance = StagingSystem.objects.get(pk=self.pk)
+        except (StagingSystem.DoesNotExist, ValueError):
+            needs_recalc = True
+        else:
+            needs_recalc = old_instance.eve_solar_system != self.eve_solar_system
+        if self.is_main:
+            StagingSystem.objects.update(is_main=False)
+        super().save(*args, **kwargs)
+        if needs_recalc:
+            self.distances.all().delete()
+            _task_calc_staging_system().delay(self.pk)
+
+
+class DistancesFromStaging(models.Model):
+    timer = models.ForeignKey(Timer, on_delete=models.CASCADE, related_name="distances")
+    staging_system = models.ForeignKey(
+        StagingSystem, on_delete=models.CASCADE, related_name="distances"
+    )
+
+    light_years = models.FloatField(null=True, default=None, blank=True)
+    jumps = models.PositiveIntegerField(null=True, default=None, blank=True)
+    updated_at = models.DateTimeField(auto_now=True)
+
+    objects = DistancesFromStagingManager()
+
+    class Meta:
+        constraints = [
+            models.UniqueConstraint(
+                fields=["timer", "staging_system"], name="fpk_distances_from_staging"
+            )
+        ]
+
+    def __str__(self) -> str:
+        return f"{self.timer}-{self.staging_system}"
+
+    def calculate(self):
+        """Calculate all distances."""
+        self.light_years = meters_to_ly(
+            self.staging_system.eve_solar_system.distance_to(
+                self.timer.eve_solar_system
+            )
+        )
+        self.jumps = self.staging_system.eve_solar_system.jumps_to(
+            self.timer.eve_solar_system
+        )
